package java.time.chrono;

import java.io.IOException;
import sun.util.calendar.BaseCalendar;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.util.HashMap;
import java.util.Objects;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.io.FileInputStream;
import java.io.File;
import java.time.format.ResolverStyle;
import java.time.temporal.TemporalField;
import java.util.Map;
import java.time.temporal.ValueRange;
import java.util.Arrays;
import java.util.List;
import java.time.Instant;
import java.time.temporal.ChronoField;
import java.time.temporal.TemporalAccessor;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.Clock;
import java.util.Iterator;
import java.time.DateTimeException;
import sun.util.logging.PlatformLogger;
import java.util.Properties;
import java.io.Serializable;

public final class HijrahChronology extends AbstractChronology implements Serializable
{
    private final transient String typeId;
    private final transient String calendarType;
    private static final long serialVersionUID = 3127340209035924785L;
    public static final HijrahChronology INSTANCE;
    private transient volatile boolean initComplete;
    private transient int[] hijrahEpochMonthStartDays;
    private transient int minEpochDay;
    private transient int maxEpochDay;
    private transient int hijrahStartEpochMonth;
    private transient int minMonthLength;
    private transient int maxMonthLength;
    private transient int minYearLength;
    private transient int maxYearLength;
    private static final transient Properties calendarProperties;
    private static final String PROP_PREFIX = "calendar.hijrah.";
    private static final String PROP_TYPE_SUFFIX = ".type";
    private static final String KEY_ID = "id";
    private static final String KEY_TYPE = "type";
    private static final String KEY_VERSION = "version";
    private static final String KEY_ISO_START = "iso-start";
    
    private static void registerVariants() {
        for (String s : HijrahChronology.calendarProperties.stringPropertyNames()) {
            if (s.startsWith("calendar.hijrah.")) {
                String substring = s.substring("calendar.hijrah.".length());
                if (substring.indexOf(46) >= 0) {
                    continue;
                }
                if (substring.equals(HijrahChronology.INSTANCE.getId())) {
                    continue;
                }
                try {
                    AbstractChronology.registerChrono(new HijrahChronology(substring));
                }
                catch (DateTimeException t) {
                    PlatformLogger.getLogger("java.time.chrono").severe("Unable to initialize Hijrah calendar: " + substring, t);
                }
            }
            else {
                continue;
            }
        }
    }
    
    private HijrahChronology(String s) throws DateTimeException {
        if (s.isEmpty()) {
            throw new IllegalArgumentException("calendar id is empty");
        }
        String string = "calendar.hijrah." + s + ".type";
        String property = HijrahChronology.calendarProperties.getProperty(string);
        if (property == null || property.isEmpty()) {
            throw new DateTimeException("calendarType is missing or empty for: " + string);
        }
        this.typeId = s;
        this.calendarType = property;
        return;
    }
    
    private void checkCalendarInit() {
        if (!this.initComplete) {
            this.loadCalendarData();
            this.initComplete = true;
        }
    }
    
    @Override
    public String getId() {
        return this.typeId;
    }
    
    @Override
    public String getCalendarType() {
        return this.calendarType;
    }
    
    @Override
    public HijrahDate date(Era era, int n, int n2, int n3) {
        return this.date(this.prolepticYear(era, n), n2, n3);
    }
    
    @Override
    public HijrahDate date(int n, int n2, int n3) {
        return HijrahDate.of(this, n, n2, n3);
    }
    
    @Override
    public HijrahDate dateYearDay(Era era, int n, int n2) {
        return this.dateYearDay(this.prolepticYear(era, n), n2);
    }
    
    @Override
    public HijrahDate dateYearDay(int n, int i) {
        HijrahDate of = HijrahDate.of(this, n, 1, 1);
        if (i > of.lengthOfYear()) {
            throw new DateTimeException("Invalid dayOfYear: " + i);
        }
        return of.plusDays((long)(i - 1));
    }
    
    @Override
    public HijrahDate dateEpochDay(long n) {
        return HijrahDate.ofEpochDay(this, n);
    }
    
    @Override
    public HijrahDate dateNow() {
        return this.dateNow(Clock.systemDefaultZone());
    }
    
    @Override
    public HijrahDate dateNow(ZoneId zone) {
        return this.dateNow(Clock.system(zone));
    }
    
    @Override
    public HijrahDate dateNow(Clock clock) {
        return this.date(LocalDate.now(clock));
    }
    
    @Override
    public HijrahDate date(TemporalAccessor temporalAccessor) {
        if ((temporalAccessor instanceof HijrahDate)) {
            return (HijrahDate)temporalAccessor;
        }
        return HijrahDate.ofEpochDay(this, temporalAccessor.getLong(ChronoField.EPOCH_DAY));
    }
    
    @Override
    public ChronoLocalDateTime<HijrahDate> localDateTime(TemporalAccessor temporalAccessor) {
        return super.localDateTime(temporalAccessor);
    }
    
    @Override
    public ChronoZonedDateTime<HijrahDate> zonedDateTime(TemporalAccessor temporalAccessor) {
        return super.zonedDateTime(temporalAccessor);
    }
    
    @Override
    public ChronoZonedDateTime<HijrahDate> zonedDateTime(Instant instant, ZoneId zoneId) {
        return super.zonedDateTime(instant, zoneId);
    }
    
    @Override
    public boolean isLeapYear(long n) {
        this.checkCalendarInit();
        return n >= (long)this.getMinimumYear() && n <= (long)this.getMaximumYear() && this.getYearLength((int)n) > 354;
    }
    
    @Override
    public int prolepticYear(Era era, int n) {
        if (!(era instanceof HijrahEra)) {
            throw new ClassCastException("Era must be HijrahEra");
        }
        return n;
    }
    
    @Override
    public HijrahEra eraOf(int n) {
        switch (n) {
            case 1: {
                return HijrahEra.AH;
            }
            default: {
                throw new DateTimeException("invalid Hijrah era");
            }
        }
    }
    
    @Override
    public List<Era> eras() {
        return Arrays.asList(HijrahEra.values());
    }
    
    @Override
    public ValueRange range(ChronoField chronoField) {
        this.checkCalendarInit();
        if ((chronoField instanceof ChronoField)) {
            switch (chronoField) {
                case DAY_OF_MONTH: {
                    return ValueRange.of(1L, 1L, (long)this.getMinimumMonthLength(), (long)this.getMaximumMonthLength());
                }
                case DAY_OF_YEAR: {
                    return ValueRange.of(1L, (long)this.getMaximumDayOfYear());
                }
                case ALIGNED_WEEK_OF_MONTH: {
                    return ValueRange.of(1L, 5L);
                }
                case YEAR:
                case YEAR_OF_ERA: {
                    return ValueRange.of((long)this.getMinimumYear(), (long)this.getMaximumYear());
                }
                case ERA: {
                    return ValueRange.of(1L, 1L);
                }
                default: {
                    return chronoField.range();
                }
            }
        }
        else {
            return chronoField.range();
        }
    }
    
    @Override
    public HijrahDate resolveDate(Map<TemporalField, Long> map, ResolverStyle resolverStyle) {
        return (HijrahDate)super.resolveDate(map, resolverStyle);
    }
    
    int checkValidYear(long lng) {
        if (lng < (long)this.getMinimumYear() || lng > (long)this.getMaximumYear()) {
            throw new DateTimeException("Invalid Hijrah year: " + lng);
        }
        return (int)lng;
    }
    
    void checkValidDayOfYear(int i) {
        if (i < 1 || i > this.getMaximumDayOfYear()) {
            throw new DateTimeException("Invalid Hijrah day of year: " + i);
        }
        return;
    }
    
    void checkValidMonth(int i) {
        if (i < 1 || i > 12) {
            throw new DateTimeException("Invalid Hijrah month: " + i);
        }
        return;
    }
    
    int[] getHijrahDateInfo(int n) {
        this.checkCalendarInit();
        if (n < this.minEpochDay || n >= this.maxEpochDay) {
            throw new DateTimeException("Hijrah date out of range");
        }
        int epochDayToEpochMonth = this.epochDayToEpochMonth(n);
        return new int[] { this.epochMonthToYear(epochDayToEpochMonth), this.epochMonthToMonth(epochDayToEpochMonth) + 1, n - this.epochMonthToEpochDay(epochDayToEpochMonth) + 1 };
    }
    
    long getEpochDay(int i, int j, int k) {
        this.checkCalendarInit();
        this.checkValidMonth(j);
        int n = this.yearToEpochMonth(i) + (j - 1);
        if (n < 0 || n >= this.hijrahEpochMonthStartDays.length) {
            throw new DateTimeException("Invalid Hijrah date, year: " + i + ", month: " + j);
        }
        if (k < 1 || k > this.getMonthLength(i, j)) {
            throw new DateTimeException("Invalid Hijrah day of month: " + k);
        }
        return (long)(this.epochMonthToEpochDay(n) + (k - 1));
    }
    
    int getDayOfYear(int n, int n2) {
        return this.yearMonthToDayOfYear(n, n2 - 1);
    }
    
    int getMonthLength(int i, int j) {
        int n = this.yearToEpochMonth(i) + (j - 1);
        if (n < 0 || n >= this.hijrahEpochMonthStartDays.length) {
            throw new DateTimeException("Invalid Hijrah date, year: " + i + ", month: " + j);
        }
        return this.epochMonthLength(n);
    }
    
    int getYearLength(int n) {
        return this.yearMonthToDayOfYear(n, 12);
    }
    
    int getMinimumYear() {
        return this.epochMonthToYear(0);
    }
    
    int getMaximumYear() {
        return this.epochMonthToYear(this.hijrahEpochMonthStartDays.length - 1) - 1;
    }
    
    int getMaximumMonthLength() {
        return this.maxMonthLength;
    }
    
    int getMinimumMonthLength() {
        return this.minMonthLength;
    }
    
    int getMaximumDayOfYear() {
        return this.maxYearLength;
    }
    
    int getSmallestMaximumDayOfYear() {
        return this.minYearLength;
    }
    
    private int epochDayToEpochMonth(int key) {
        int binarySearch = Arrays.binarySearch(this.hijrahEpochMonthStartDays, key);
        if (binarySearch < 0) {
            binarySearch = -binarySearch - 2;
        }
        return binarySearch;
    }
    
    private int epochMonthToYear(int n) {
        return (n + this.hijrahStartEpochMonth) / 12;
    }
    
    private int yearToEpochMonth(int n) {
        return n * 12 - this.hijrahStartEpochMonth;
    }
    
    private int epochMonthToMonth(int n) {
        return (n + this.hijrahStartEpochMonth) % 12;
    }
    
    private int epochMonthToEpochDay(int n) {
        return this.hijrahEpochMonthStartDays[n];
    }
    
    private int yearMonthToDayOfYear(int n, int n2) {
        int yearToEpochMonth = this.yearToEpochMonth(n);
        return this.epochMonthToEpochDay(yearToEpochMonth + n2) - this.epochMonthToEpochDay(yearToEpochMonth);
    }
    
    private int epochMonthLength(int n) {
        return this.hijrahEpochMonthStartDays[n + 1] - this.hijrahEpochMonthStartDays[n];
    }
    
    private static Properties readConfigProperties(String s) throws Exception {
        try {
            return (Properties)AccessController.doPrivileged(()/*  */ -> {
                File file = new File(System.getProperty("java.home") + File.separator + "lib", p0);
                Properties properties = new Properties();
                FileInputStream inStream = new FileInputStream(file);
                try {
                    properties.load(inStream);
                    if (inStream != null) {
                        Throwable t;
                        if (t != null) {
                            try {
                                inStream.close();
                            }
                            catch (Throwable exception) {
                                t.addSuppressed(exception);
                            }
                        }
                        else {
                            inStream.close();
                        }
                    }
                }
                catch (Throwable t2) {
                    throw t2;
                }
                finally {
                    if (inStream != null) {
                        Throwable t;
                        if (t != null) {
                            try {
                                inStream.close();
                            }
                            catch (Throwable exception2) {
                                t.addSuppressed(exception2);
                            }
                        }
                        else {
                            inStream.close();
                        }
                    }
                }
                return properties;
            });
        }
        catch (PrivilegedActionException ex) {
            throw ex.getException();
        }
    }
    
    private void loadCalendarData() {
        try {
            String property = HijrahChronology.calendarProperties.getProperty("calendar.hijrah." + this.typeId);
            Objects.requireNonNull(property, "Resource missing for calendar: calendar.hijrah." + this.typeId);
            Properties configProperties = readConfigProperties(property);
            HashMap<Integer, int[]> hashMap = new HashMap();
            int i = Integer.MAX_VALUE;
            int max = Integer.MIN_VALUE;
            String s = null;
            String s2 = null;
            String s3 = null;
            int minEpochDay = 0;
            for (Map.Entry entry : configProperties.entrySet()) {
                String s4 = (String)entry.getKey();
                int n = -1;
                switch (s4.hashCode()) {
                    case 3355: {
                        if (s4.equals("id")) {
                            n = 0;
                            break;
                        }
                        break;
                    }
                    case 3575610: {
                        if (s4.equals("type")) {
                            n = 1;
                            break;
                        }
                        break;
                    }
                    case 351608024: {
                        if (s4.equals("version")) {
                            n = 2;
                            break;
                        }
                        break;
                    }
                    case -1117701862: {
                        if (s4.equals("iso-start")) {
                            n = 3;
                            break;
                        }
                        break;
                    }
                }
                switch (n) {
                    case 0: {
                        s = (String)entry.getValue();
                        continue;
                    }
                    case 1: {
                        s2 = (String)entry.getValue();
                        continue;
                    }
                    case 2: {
                        s3 = (String)entry.getValue();
                        continue;
                    }
                    case 3: {
                        int[] ymd = this.parseYMD((String)entry.getValue());
                        minEpochDay = (int)LocalDate.of(ymd[0], ymd[1], ymd[2]).toEpochDay();
                        continue;
                    }
                    default: {
                        try {
                            int intValue = (int)Integer.valueOf(s4);
                            hashMap.put(Integer.valueOf(intValue), this.parseMonths((String)entry.getValue()));
                            max = Math.max(max, intValue);
                            i = Math.min(i, intValue);
                            continue;
                        }
                        catch (NumberFormatException ex) {
                            throw new IllegalArgumentException("bad key: " + s4);
                        }
                    }
                }
            }
            if (!this.getId().equals(s)) {
                throw new IllegalArgumentException("Configuration is for a different calendar: " + s);
            }
            if (!this.getCalendarType().equals(s2)) {
                throw new IllegalArgumentException("Configuration is for a different calendar type: " + s2);
            }
            if (s3 == null || s3.isEmpty()) {
                throw new IllegalArgumentException("Configuration does not contain a version");
            }
            if (minEpochDay == 0) {
                throw new IllegalArgumentException("Configuration does not contain a ISO start date");
            }
            this.hijrahStartEpochMonth = i * 12;
            this.minEpochDay = minEpochDay;
            this.hijrahEpochMonthStartDays = this.createEpochMonths(this.minEpochDay, i, max, hashMap);
            this.maxEpochDay = this.hijrahEpochMonthStartDays[this.hijrahEpochMonthStartDays.length - 1];
            for (; i < max; ++i) {
                int yearLength = this.getYearLength(i);
                this.minYearLength = Math.min(this.minYearLength, yearLength);
                this.maxYearLength = Math.max(this.maxYearLength, yearLength);
            }
        }
        catch (Exception ex2) {
            PlatformLogger.getLogger("java.time.chrono").severe("Unable to initialize Hijrah calendar proxy: " + this.typeId, ex2);
            throw new DateTimeException("Unable to initialize HijrahCalendar: " + this.typeId, ex2);
        }
    }
    
    private int[] createEpochMonths(int n, int i, int n2, Map<Integer, int[]> map) {
        int n3 = (n2 - i + 1) * 12 + 1;
        int j = 0;
        int[] array = new int[n3];
        this.minMonthLength = Integer.MAX_VALUE;
        this.maxMonthLength = Integer.MIN_VALUE;
        int[] array2;
        int k;
        int n4;
        for (; i <= n2; ++i) {
            array2 = (int[])map.get(Integer.valueOf(i));
            k = 0;
            while (k < 12) {
                n4 = array2[k];
                array[j++] = n;
                if (n4 < 29 || n4 > 32) {
                    throw new IllegalArgumentException("Invalid month length in year: " + i);
                }
                n += n4;
                this.minMonthLength = Math.min(this.minMonthLength, n4);
                this.maxMonthLength = Math.max(this.maxMonthLength, n4);
                ++k;
            }
        }
        array[j++] = n;
        if (j != array.length) {
            throw new IllegalStateException("Did not fill epochMonths exactly: ndx = " + j + " should be " + array.length);
        }
        return array;
    }
    
    private int[] parseMonths(String s) {
        int[] array = new int[12];
        String[] split = s.split("\\s");
        if (split.length != 12) {
            throw new IllegalArgumentException("wrong number of months on line: " + Arrays.toString(split) + "; count: " + split.length);
        }
        for (int i = 0; i < 12; ++i) {
            try {
                array[i] = (int)Integer.valueOf(split[i]);
            }
            catch (NumberFormatException ex) {
                throw new IllegalArgumentException("bad key: " + split[i]);
            }
        }
        return array;
    }
    
    private int[] parseYMD(String trim) {
        trim = trim.trim();
        try {
            if (trim.charAt(4) != '-' || trim.charAt(7) != '-') {
                throw new IllegalArgumentException("date must be yyyy-MM-dd");
            }
            return new int[] { (int)Integer.valueOf(trim.substring(0, 4)), (int)Integer.valueOf(trim.substring(5, 7)), (int)Integer.valueOf(trim.substring(8, 10)) };
        }
        catch (NumberFormatException cause) {
            throw new IllegalArgumentException("date must be yyyy-MM-dd", cause);
        }
    }
    
    @Override
    Object writeReplace() {
        return super.writeReplace();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Deserialization via serialization delegate");
    }
    
    static {
        try {
            calendarProperties = BaseCalendar.getCalendarProperties();
        }
        catch (IOException cause) {
            throw new InternalError("Can't initialize lib/calendars.properties", cause);
        }
        try {
            AbstractChronology.registerChrono(INSTANCE = new HijrahChronology("Hijrah-umalqura"), "Hijrah");
            AbstractChronology.registerChrono(HijrahChronology.INSTANCE, "islamic");
        }
        catch (DateTimeException t) {
            PlatformLogger.getLogger("java.time.chrono").severe("Unable to initialize Hijrah calendar: Hijrah-umalqura", t);
            throw new RuntimeException("Unable to initialize Hijrah-umalqura calendar", t.getCause());
        }
        registerVariants();
    }
}
